#!/bin/bash
# Copyright (C) 2017-2021 sāf.ai, Inc. (https://saf.ai)
# All rights reserved.
#

# Common shell functions.

die() {
	local rv=$1
	shift
	(printf "%s\n" "[ERROR] $@") 1>&2
	exit $rv
}

warn() {
	(echo "[WARNING] $@") 1>&2
}

info() {
	(echo "[INFO] $@") 1>&2
}

BUG() {
	local line TRACE

	for i in 9 8 7 6 5 4 3 2 1 0; do
		line=$(caller $i)
		if [ -z ${line:+X} ]; then
			continue
		fi
		line="FROM ${line}"
		TRACE=$(echo -e "${TRACE:+${TRACE}\n...}$line")
	done
cat 1>&2 <<EOF
[BUG]:: $@
	You have discovered an internal problem with cefs package-manager
	Please help us correct it by filing a report, or sending us an email to
	support@saf.ai!
--- Trace ---
$TRACE
--- End of trace
EOF
	exit 127
}

is_func() {
	local a=$(type -t "$1")
	test ${a:-X} = "function"
}

need_cmd() {
	if ! check_cmd "$1" > /dev/null 2>&1; then
		die 2 "need '$1' (command not found)"
	fi
}

check_cmd() {
	command -v "$1"
}

log() {
	case "$1" in
	INIT)
		if mkdir -p $(dirname ${LOGFILE}) ; then
			echo "Using ${LOGFILE}" 1>&2
			exec 5>${LOGFILE}
		else
			exec 5>&2
		fi
		;;
	EXEC)
		shift
		echo "# $@" >&5
		"$@" 1>&5 2>&5
		;;
	*)
		echo "$@" 1>&5
		;;
	esac
}

# Run a command that should never fail. If the command fails execution
# will immediately terminate with an error showing the failing command.
ensure() {
	if ! await_status "$@" ; then
		die 9 "Failed to execute\n\t$*\n(see $LOGFILE for details)"
	fi
}

status() {
	case "$1" in
	MSG)
		shift
		local msg="$@"
		printf "%s" "$msg"
		# eval is needed by bash
		if [[ ${#msg} -lt 70 ]]; then
			eval printf ".%.0s" {1..$((70 - ${#msg}))}
		fi
		;;
	OK|FAIL|SKIP)
		printf "[ %-6s ]" $1
		;;
	END)
		printf "\n"
		;;
	esac
}

await_status() {
	if [ "$1" = "-m" ]; then
		msg="$2"
		status MSG $msg
		shift; shift
	elif is_func $1:msg ; then
		status MSG $($1:msg $*)
	else
		status MSG "$1"
	fi

	if [ $# -eq 0 ]; then
		BUG "await_status used with out a command"
	fi

	if log EXEC "$@" ; then status OK ; else status FAILED; fi

	status END
}

init_vars() {
	P="${0}"
	P=$(realpath ${P})
	D=$(dirname ${0})
	## --- Configuration variables ---

	export prefix=${prefix:-/usr}
	export cefsroot=${prefix}/lib/cefs/1.9.9-rc1
	export bindir=${cefsroot}/bin
	export libdir=${cefsroot}/lib
	export sysconfdir=/etc/cefs
	export localstatedir=/var/run/cefs
	export libexecdir=${cefsroot}/libexec
	export syslibdir=/var/lib/cefs
	export logdir=/var/log/cefs
	export innerrootdir=root
	export installroot=${installroot:-${prefix}}
	export PACKAGE_NAME=cefs
	export PACKAGE_VERSION=1.9.9-rc1
	export TARBALL_URL=https://get.saf.ai/beta/ubuntu-22.04/cefs-1.9.9-rc1-ubuntu-22.04.tar.xz
	export TARBALL_BASENAME=cefs-1.9.9-rc1-ubuntu-22.04.tar.xz
	export SETUP_SH_BASENAME=cefs-installer.sh
	export CEFS_BIN=${cefsroot}/bin/cefs

	## --- Configuration variables: END ---

	return 0
}



main() {

	init_vars

	PACKAGE_NAME="cefs"
	PACKAGE_VERSION="1.9.9-rc1"

	test -f /usr/lib/${PACKAGE_NAME}/${PACKAGE_VERSION}/settings && \
		. /usr/lib/${PACKAGE_NAME}/${PACKAGE_VERSION}/settings

	export PATH="$cefsroot/bin:$cefsroot/sbin:$PATH"
	export LD_LIBRARY_PATH="$cefsroot/lib:$LD_LIBRARY_PATH"
    ulimit -v unlimited
    ulimit -c 512000
    ulimit -l unlimited
    ulimit -s unlimited
    ulimit -n 1048576
	unset LANG

	## Immunize against OOM kills and sendsigs signals
	if [ -f "/proc/$$/oom_score_adj" ]
	then
		echo -1000 > "/proc/$$/oom_score_adj"
	else
		echo -17 > "/proc/$$/oom_adj"
	fi

	if [ "$1" = "filesystem" -a "$2" = "mount" ]; then
		LOG_LEVEL=${CEFS_LOG_LEVEL:-info}
	else
		LOG_LEVEL=${CEFS_CMD_LOG_LEVEL:-warn}
	fi
	export CEFS_LOG=${LOG_LEVEL}
	exec ${CEFS_BIN} "$@"
}

main "$@"
